import pytest
from StataHelper import StataHelper
from wrappers import parallelize
from utils import limit_cores
import numpy as np
from multiprocessing import cpu_count
from builtins import *

params = {'y': ['mpg'], 'x': [['weight', 'length'], ['weight']]}
statapath = r"C:\Program Files\Stata18\utilities"
cmd = "regress {y} {x}"
estimatesdir = "D:\Collin\statahelper"
s = StataHelper(stata_path=statapath, edition='mp', splash=False,  set_output_dir=estimatesdir)
s.schedule(cmd, params)

def add(x, y):
    return x + y

def test_paralleize():
    assert parallelize(add, [(1, 2), (3, 4), (5, 6)]) == [3, 7, 11]

def test_parallelize_args():
    def p(func,iterable, *args):
        if args:
            iterable = [(i, *args) for i in iterable]
        return iterable
    assert p(add, [(1, 2), (3, 4), (5, 6)], 1) == [((1,2),1), ((3,4),1), ((5,6),1)]

def test_params_kwargs():
    kwargs = {'quetly': True}
    params = [(i, kwargs) for i in s.queue]
    assert params == [("regress mpg weight length", {'quetly': True}),
                              ("regress mpg weight", {'quetly': True})]

def test_params_no_kwargs():
    params = [i for i in s.queue]
    assert params == ["regress mpg weight length",
                              "regress mpg weight"]

def test_limit_maxcores():
    it = np.arange(1, 10)
    assert limit_cores(it, 2) == 2

def test_limit_big_iterable():
    it = np.arange(0, 100)
    assert limit_cores(it) == cpu_count()-1

def test_limit_small_iterable():
    it = np.arange(0, 4)
    assert limit_cores(it) == 4

def test_parallel_name():
    cmd = "regress {y} {x}\nestimates store *_{y}_{x}"
    pmap = {'y': ['mpg'], 'x': [['weight', 'length'], ['weight']]}
    name = 'test'
    kwargs = {'quetly': True}
    queue = s.schedule(cmd, pmap)
    params = list(enumerate([i for i in queue]))
    params = [(i, j) for i, j in params]
    assert params == [(0, 'regress mpg weight length'), (1, 'regress mpg weight')]

def test_parallel_name_kwargs():
    cmd = "regress {y} {x}\nestimates store *_{y}_{x}"
    pmap = {'y': ['mpg'], 'x': [['weight', 'length'], ['weight']]}
    name = 'test'
    kwargs = {'quetly': True}
    queue = s.schedule(cmd, pmap)
    params = list(enumerate([(i, kwargs) for i in queue]))
    params = [(i, j[0], j[1]) for i, j in params]
    assert params == [(0, 'regress mpg weight length', {'quetly': True}),
                      (1, 'regress mpg weight', {'quetly': True})]


import pytest
from StataHelper import StataHelper
from builtins import ValueError

params = {'y': ['mpg'], 'x': [['weight', 'length'], ['weight']]}
statapath = r"C:\Program Files\Stata18\utilities"
s = StataHelper(stata_path=statapath, edition='mp', splash=False)

def test_stata_init():
    assert s is not None

def test_stata_init_path():
    assert s.stata_path == r"C:\Program Files\Stata18\utilities"

def test_stata_init_path_error():
    with pytest.raises(ValueError):
        s1= StataHelper(stata_path='C:/Program Files/Stata17/StataMP-64.exe',
                        edition='mp', splash=False)

def test_is_initialized():
    assert s.is_stata_initialized() == True




import pytest
from utils import *


def cartesian_dict_happy():
    """
    Test the cartesian function with a dictionary
    """
    args = {
        'a': [[1, 2], [3, 4]],
        'b': [3, 4]
    }
    assert cartesian(args) == [
        ([1, 2], 3),
        ([1, 2], 4),
        ([3, 4], 3),
        ([3, 4], 4)
    ]


def cartesian_list_happy():
    """
    Test the cartesian function with a list
    """
    args = [
        [1, 2],
        [3, 4]
    ]
    assert cartesian(args) == [
        (1, 3),
        (1, 4),
        (2, 3),
        (2, 4)
    ]


def cartesian_sad():
    """
    Test the cartesian function with a string
    """
    args = 'a string'
    with pytest.raises(ValueError):
        cartesian(args)


def read_yaml_happy():
    """
    Test the read_yaml function with a string
    """
    yamlpath = 'tests/resources/test.yaml'
    assert read_yaml(yamlpath) == {
        'quiet': False,
        'y': None,
        'x': None,
        'fe1': None,
        'fe2': None,
        'arg1': None
        }


def read_yaml_sad():
    """
    Test the read_yaml function with a string
    """
    yamlpath = 123
    with pytest.raises(ValueError) as exp:
        read_yaml(yamlpath)
        assert str(exp.value) == "expected type str or dict, got <class 'int'>"


def read_yaml_file_not_found():
    """
    Test the read_yaml function with a string
    """
    yamlpath = 'tests/resources/not_a_file.yaml'
    with pytest.raises(FileNotFoundError):
        read_yaml(yamlpath)


def config_yaml_happy():
    """
    Test the config_yaml function
    """
    assert config_yaml() == {
        'quiet': False,
        'y': None,
        'x': None,
        'fe1': None,
        'fe2': None,
        'arg1': None
        }


def read_keys_happy():
    """
    Test the read_keys function with a string
    """
    string = 'quiet'
    dictionary = {
        'quiet': False,
        'y': None,
        'x': None,
        'fe1': None,
        'fe2': None,
        'arg1': None
        }
    assert read_keys(string, dictionary) == False


def read_keys_sad():
    """
    Test the read_keys function with a string
    """
    string = 'not_in_dict'
    dictionary = {
        'quiet': False,
        'y': None,
        'x': None,
        'fe1': None,
        'fe2': None,
        'arg1': None
        }
    assert read_keys(string, dictionary) is None



import pytest
from builtins import ValueError

from StataHelper import StataHelper

params = {'y': ['mpg'], 'x': [['weight', 'length'], ['weight']]}
statapath = r"C:\Program Files\Stata18\utilities"
cmd = "regress {y} {x}\nestimates store *_{y}_{x}"
estimatesdir = "D:\Collin\statahelper"
dta = f'{statapath.replace("utilities", "auto.dta")}'

s = StataHelper(stata_path=statapath, edition='mp', splash=True,  set_output_dir=estimatesdir)

def test_load():
    s.use(dta)# add assertion here

def test_columns_list():
    s.use(dta, columns=['mpg', 'weight'])

def test_columns_string():
    s.use(dta, columns='mpg weight')

def test_columns_list_error():
    with pytest.raises(ValueError):
        s.use(dta, columns=1)

def test_obs1():
    s.use(dta, obs=10)


def test_obs2():
    s.use(dta, obs="10")


def test_obs3():
    s.use(dta, obs="1/10")


def test_obs_cols():
    s.use(dta, obs="1/10", columns=['mpg', 'weight'])




# Test functions that send data from pandas dataframes to Stata

def test_use_file():
    pass

def test_use_file_error():
    with pytest.raises(ValueError):
        s.use_file(dta)


def test_use_as_pandas():
    s.use(dta)
    df = s.use_as_pandas()
    assert df.shape == (74, 12)

def test_use_as_pandas_frame():
    s.use(dta)
    df = s.use_as_pandas(frame="newframe")
    assert df.shape == (74, 12)


def test_use_as_pandas_error():
    with pytest.raises(ValueError):
        s.use_as_pandas()


def test_save():
    s.save(dta)

def test_save_error():
    with pytest.raises(ValueError):
        s.save(1)



from setuptools import setup, find_packages
setup(
    name='StataHelper',
    version='1.0.0',
    packages=find_packages(exclude=['tests*']),
    url='https://github.com/ColZoel/StataHelper',
    license='MIT',
    author='Collin Zoeller',
    author_email='zoellercollin@gmail.com',
    description='Simple parallelization wrapper over Pystata',
    keywords=['Stata', 'Pystata', 'parallelization', 'wrapper'],
)



from builtins import *
import pytest
from StataHelper import StataHelper
from utils import cartesian

params1 = {'y': ['mpg'], 'x': [['weight', 'length'], ['weight']]}
params2 = {'y': ['mpg'], 'x': ['weight', 'length'], 'z': ['displacement']}
cmd1 = "regress {y} {x}"
cmd2 = "regress {y} {x} {z}"
statapath = r"C:\Program Files\Stata18\utilities"
s = StataHelper(stata_path=statapath, edition='mp', splash=False)

def test_notequal_sad1():
    with pytest.raises(ValueError):
        q = s.schedule(cmd2, params1)

    with pytest.raises(ValueError):
        q = s.schedule(cmd1, params2)

def test_cartesian():
    assert cartesian(params1.values()) == [('mpg', 'weight'), ('mpg', ['weight', 'length'])]

def test_process_map():
    cartesian_args = cartesian(params1.values())
    process_maps = [dict(zip(params1.keys(), c)) for c in cartesian_args]
    assert process_maps == [{'y': 'mpg', 'x': 'weight'}, {'y': 'mpg', 'x': ['weight', 'length']}]

def test_queue():
    cartesian_args = cartesian(params1.values())
    process_maps = [dict(zip(params1.keys(), c)) for c in cartesian_args]
    queue = [s._parse_cmd(cmd1, i) for i in process_maps]
    assert queue == ['regress mpg weight', 'regress mpg weight length']

def test_schedule():
    q = s.schedule(cmd1, params1)
    expected = ['regress mpg weight length', 'regress mpg weight']
    assert q == expected